#pragma once

#include <random>
#include "kwargs.hpp"

const Kwarg<uint32_t> id;

template<class super>
class header_id : public super
{
    typedef typename super::buffer_type buffer;
    typedef uint32_t id_type;

    struct header_id_type {
        id_type id;
    };

    constexpr static size_t m_header_id_len = sizeof(header_id_type);
    id_type m_id_out;
    id_type m_id_in;

    struct header_id_type *cast_header(buffer &buf)
    {
        return reinterpret_cast<struct header_id_type *>(buf.head());
    }

    void write_id(buffer &buf)
    {
        buf.head_add(m_header_id_len);
        cast_header(buf)->id = htobe32(m_id_out);
    }

    void read_id(buffer &buf)
    {
        m_id_in = be32toh(cast_header(buf)->id);
        buf.head_del(m_header_id_len);
    }

  public:
    template<typename... Args> explicit
    header_id(const Args&... args) : super(args...)
    {
        kwget(id, m_id_out, args...);
    }

    static id_type get_random_id()
    {
        auto rnd = std::default_random_engine(std::random_device{}());

        return rnd();
    }

    bool recv_buf(buffer &buf)
    {
        if (!super::recv_buf(buf))
            return false;

        read_id(buf);

        return true;
    }

    bool send_buf(buffer &buf)
    {
        write_id(buf);

        return super::send_buf(buf);
    }

    id_type id_out() const
    {
        return m_id_out;
    }

    id_type id_in() const
    {
        return m_id_in;
    }
};
