#ifndef KWARGS_HPP
#define KWARGS_HPP

// Ross Smith (rosssmith@me.com)
// This code is in the public domain

#include <type_traits>

namespace KwargsImpl {

    template <typename Src, typename Dst,
        bool = std::is_convertible<Src, Dst>::value>
    struct Copy {
        void operator()(const Src& src, Dst& dst) const { dst = src; }
    };

    template <typename Src, typename Dst>
    struct Copy<Src, Dst, false> {
        void operator()(const Src&, Dst&) const {}
    };

    template <typename Val>
    struct Param {
        const void* key;
        Val value;
    };

}

template <typename Val>
struct Kwarg {
    Kwarg() {}
    template <typename Arg> KwargsImpl::Param<Val>
        operator=(const Arg& arg) const { return {this, arg}; }
};

template <typename Key, typename Val, typename Par, typename... Args>
void kwget(const Kwarg<Key>& key, Val& value,
        const KwargsImpl::Param<Par>& param, const Args&... more) {
    if (&key == param.key)
        KwargsImpl::Copy<Par, Val>()(param.value, value);
    kwget(key, value, more...);
}

template <typename Key, typename Val, typename... Args>
void kwget(const Kwarg<Key>& key, Val& value,
        const Kwarg<bool>& param, const Args&... more) {
    kwget(key, value, param = true, more...);
}

template <typename Key, typename Val>
void kwget(const Kwarg<Key>&, Val&) {}

#endif
